using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using System.Text;
using System.Globalization;
using System.Runtime.InteropServices;
using System.Text.RegularExpressions;
using System.IO;
using System.Timers;
using System.Threading;
using System.Diagnostics;


namespace Super882
{    
	public enum retCode : int
	{
		/**************************************************************************
		* R E T U R N   S T A T U S   C O D E S		0x0..
		**************************************************************************/
		 stat_OK					=0x00,	// status OK
		 stat_NO_TAG_ERR			=0x01,	// no tag error		 	
		 stat_COLL_ERR				=0x02,	// collision error				
		 stat_AUTH_ERR				=0x03,	// mifare authentication error		
		 stat_PROTOCOL_ERR			=0x04,	// T=CL protocol error
		 stat_TRANSMISSION_ERR		=0x05,	// T=CL transmission error
		 stat_TIMEOUT_ERR			=0x06,	// T=CL timeout error
		 stat_BUFFER_OVERFLOW_ERR	=0x07,	// buffer overflow error
		 stat_ADR_OVERFLOW_ERR		=0x08,	// address overflow error
		 stat_UNKNOWN_CMD			=0x09,	// unknown command
		 stat_ERROR					=0x0A,	// uncategorized error
		 stat_COMM_TIMEOUT			=0x0B,	// communication timeout
		 stat_USB_DEVICE_ERROR		=0x0C,	// usb device error
		 stat_BOOT_ERROR			=0x0D,	// boot error
		 stat_BOOT_OVERFLOW			=0x0E,	// boot overflow error
		 stat_BOOT_TIMEOUT			=0x0F,	// boot timeout error
		 stat_NO_USER_CODE			=0x10,	// no user code exists
		 stat_INVALID				=0x11,	// invalid 
		 stat_NO_DATA				=0x12,	// No magnetic data decoded
		 stat_UNAVAILABLE			=0x13,	// cannot perform this command
		 stat_MAG_DATA_NOT_VALID	=0x14,	// magdata not valid
		 stat_CARD_JAM				=0x15,	// card jam
		/**************************************************************************
		* E V E N T   S T A T U S   C O D E S		0x3..
		**************************************************************************/
		 event_REMOVED				=0x30,	// contactless tag is removed
		 event_PICC_ACK				=0x31,	// contactless tag is detected
		 event_USER_MODE_START		=0x32,	// user mode started
		 event_BOOT_MODE_START		=0x33,	// boot mode started
		 event_SWITCH_REPORT		=0x34,	// automatic switch report
		 event_MEM_CARD_ACTIVATED	=0x35,	// memory card is activated	
		 event_CHIP_CARD_ACTIVATED	=0x36,	// auto atr
		 event_MDB_DATA				=0x37,	// MDB data packet
		 event_MDB_STATE			=0x38,	// MDB state change
		 event_KEY_STROKE			=0x39,	// Key entered
		 event_ARM_ACK				=0x3A,	// Arm to read ack
		 event_ARM_NACK				=0x3B,	// Arm to read nack
		 event_NO_MAG_CARD			=0x3C,	// No magnetic tracks detected
		 event_INTAKE_ACK			=0x3D,	// Intake to position ack	
		 event_INTAKE_NACK			=0x3E	// Intake to position not ack	
	};

	public enum cmdCode : int
	{
		/**************************************************************************
		* Contactless ISO14443 PCD Commands
		**************************************************************************/
		PCD_KILL				=0x1F,	// Reset PCD	
		PCD_TYPEA_INIT			=0x20,	// Init PCD	
		PCD_TYPEB_INIT			=0x50,	// Initialize Type B interface
		PCD_WRITEE2				=0x21,	// Get data from FIFO and write it to the E2PROM
		PCD_READE2				=0x22,	// Read data from E2PROM and put it into the FIFO
		PCD_RESETPHASE			=0x23,	// Runs the Reset- and Initialisation Phase
		PCD_RF_RESET			=0x25,	// Rf reset
		PCD_RF_OFF				=0x26,	// Rf off
		WRITERC					=0x57,	// Write rc
		READRC					=0x58,	// Read rc
		PCD_SET_ATTRIB			=0x29,	// Set attribute
		PICC_EXCHANGE_BLOCK		=0x2A,	// Exchange block 
		PCD_ACTIVATE_ANT		=0x61,	// Activate antenna 1 or 2
		PICC_REMOVE				=0x62,	// Remove a picc
		/**************************************************************************
		* Contactless ISO14443 TypeA mifare Commands
		**************************************************************************/
		PICC_AUTHENT			=0x13,	// Authentication
		PICC_AUTHENTKEY			=0x14,	// Authentication with extern keys
		PICC_READ				=0x15,	// Read 16 byte block
		PICC_WRITE				=0x16,	// Write 16 byte block
		PICC_WRITE4				=0x17,	// Write 4 byte block
		PICC_VALUE				=0x18,	// Decrement value
		PICC_TRANSFER			=0x1B,	// Transfer command code
		PCD_LOADKEY				=0x1D,	// Read a master key from the FIFO and put it into the master key buffer 	
		PCD_LOADKEYE2			=0x1E,	// Read a master key from the E2PROM and put it into the master key buffer
		/**************************************************************************
		* Contactless ISO14443 TypeA Commands
		**************************************************************************/
		PICC_REQUEST			=0x10,	// Request idle
		PICC_ANTICOLL			=0x11,	// Anticollision
		PICC_SELECT				=0x12,	// Select given card
		GETCARDID				=0x24,	// This command performs the Req,Anticoll, and select operations
		PICC_ANTICOLLSEL		=0x19,	// Anticollselect
		PICC_HALTA				=0x1C,	// Halt
		PICC_ACTIVATE_IDLE		=0x59,	// Activate idle card
		PICC_ACTIVATE_WAKEUP	=0x5A,	// Wake up a halted card
		PICC_ACTIVATE_AB_WAKEUP	=0x7C,	// Test command
		PICC_DO_PPS				=0x2B,	// Protocol Parameter Selection
		PICC_REQUEST_ATS		=0x3A,	// Request ATS
		/**************************************************************************
		* Contactless ISO14443 TypeB Commands
		**************************************************************************/
		PICC_REQUESTB			=0x51,	// Request or wake up a Type B picc
		PICC_SLOTMARKER			=0x5F,	// Slot Marker
		PICC_ATTRIB				=0x52,	// Attrib
		PICC_ATTRIB_HBR			=0x2C,	// Attrib Higher Bautrate
		PICC_HALTB				=0x53,	// Halt Type B picc
		/**************************************************************************
		* Contactless ISO14443-4 T=CL Commands
		**************************************************************************/
		PICC_DETECT				=0x5B,	// Detect picc
		PICC_ACTIVATE			=0x5C,	// Activate picc
		PICC_SEND_BLOCK			=0x54,	// Send block to the picc
		PICC_SEND_ACK			=0x55,	// Send ack or nack
		PICC_SEND_REQ			=0x56,	// Send WTX or DSEL
		PICC_DESELECT			=0x86,	// Perform Deselect
		/**************************************************************************
		* Reader Commands
		**************************************************************************/
		MAINTENANCE				=0x27,	// Maintenance command
		READ_STATUS				=0x28,	// Transmits reader status bytes
		READ_SWITCH				=0x2D,	// Read switch status
		READ_REGISTER			=0x30,	// Read module register
		WRITE_REGISTER			=0x31,	// Update module register
		MODULE_RESET			=0x33,	// Reset
		SET_RUNNING_MODE		=0x5D,	// Set running mode
		GET_INFO				=0x72,	// Get module information data
		/**************************************************************************
		* Peripheral Function Commands
		**************************************************************************/
		WRITE_LCD				=0x32,	// Update dot-matrix lcd
		ACT_RELAY				=0x40,	// Activate relay
		GET_INPUT				=0x87,	// Scan input pins
		SET_RELAY				=0x88,	// Set relay outputs
		SET_LED					=0x41,	// Set or clear RED/GREEN leds
		SET_BUZZER				=0x42,	// Set buzzer
		RTC_UPDATE				=0x44,	// Update date time
		SET_SHUTTER				=0x45,	// Set shutter
		MEMORY_WRITE			=0x46,	// Flash write
		MEMORY_READ				=0x47,	// Flash read
        FRM_MEM_WRITE			=0x34,	// Program firmware data
        FRM_MEM_READ			=0x35,	// Read firmware data
        FRM_MEM_ERASE           =0x36,  // Erase firmware
		MEMORY_ERASE			=0x4D,	// Flash erase
		RTC_READ				=0x48,	// Read date time info
		SET_PPLEDS				=0x5E,	// Set indicator leds
		INDICATE_PP				=0x60,	// Visual and audio indicators
		SET_AC					=0x71,	// Set access control bit
		GRAPHIC_LCD_INIT		=0x73,	// Init graphic lcd
		GRAPHIC_LCD_GOTO		=0x74,	// goto dot position
		GRAPHIC_LCD_WRITE		=0x75,	// write on graphic lcd
		GRAPHIC_LCD_SETDOT		=0x76,	// set dot
		GRAPHIC_LCD_CLRDOT		=0x77,	// clear dot
		GRAPHIC_SAVE_IMAGE		=0x78,	// save 1K image in ext. flash
		GRAPHIC_LOAD_IMAGE		=0x79,	// load 1K image data direct
		GRAPHIC_CALL_IMAGE		=0x7A,	// call a image form ext. flash
		GRAPHIC_CLEAR_IMAGES	=0x7B,	// erase all saved images
		EXTENDED				=0x43,	// Extended commands
		/**************************************************************************
		* Access Control Function Commands
		**************************************************************************/
		GET_MEM_INFO			=0x49,	// Get memory status for AC
		SAVE_LIST				=0x4A,	// Save ID list
		GET_LIST				=0x4B,	// Get ID list
		GET_ACTIVITY_FILE		=0x4C,	// Get activity file
		ERASE_ACTIVITY_FILE		=0x65,	// Erase activity file
		ERASE_ID				=0x66,	// Erase an ID from the list
		ACK						=0x67,	// Send ack
		NACK					=0x68,	// Send nack
		/**************************************************************************
		* MDB Commands
		**************************************************************************/
		SET_AVAILABLE_FUND		=0x7D,	// MDB set fund
		SET_VENDING_STATUS		=0x7E,	// MDB set vending status
		GET_VMC_CONFIG			=0x7F,	// MDB get cmv configuration
		/**************************************************************************
		* Keypad Commands
		**************************************************************************/
		KEYPAD_VERSION			=0x80,	// Read keypad Version
		EN_KEYPAD				=0x81,	// Enable keypad scan
		DIS_KEYPAD				=0x82,	// Disable keypad scan
		GET_KEYSTROKES			=0x83,	// get keystrokes
		EN_AUTOREPORT			=0x84,	// enable auto report
		DIS_AUTOREPORT			=0x85,	// disable auto report
		/**************************************************************************
		* 900 Dispenser Commands
		**************************************************************************/
		DISPENSER_DO_MOVE		=0x90,	// Move a card to the defined position
		DISPENSER_SWITCH_STAT	=0x91,	// Read switch status
		DISPENSER_MAINT			=0x92,	// Maintenance command
		/**************************************************************************
		* Bootloader Commands
		**************************************************************************/
		BOOT_ERASE				=0xA0,	// Erase user code 
		BOOT_BLANK_CHECK		=0xA1,	// Blank check
		BOOT_PROGRAM			=0xA2,	// Program flash
		BOOT_READ				=0xA3,	// Read from flash
		BOOT_VERSION			=0xA4,	// Read Version
		BOOT_RESET				=0xA5,	// Reset
		BOOT_EXE_USER_CODE		=0xA6,	// Jump to user code
		BOOT_CHECK_USER_CODE	=0xA7,	// Check user codes exists?
		BOOT_EXE_BOOT_CODE		=0xA8,	// Jump to boot code
		GET_MODE				=0xA9,	// Get current mode
		/**************************************************************************
		* Magstripe Read/Write Commands
		**************************************************************************/
		MAG_READ				=0xB0,	// Read magstripe
		MAG_READ_DETAILED		=0xB1,	// Read detailed
		MAG_GET_ISO_TRK			=0xB2,	// Transmit iso track
		MAG_GET_ISO_FMT			=0xB3,	// Transmit ISO formated data
		MAG_GET_CUSTOM_TRK		=0xB4,	// Transmit custom formated data
		MAG_ERROR				=0xB5,	// Transmit error data
		MAG_WRITE				=0xB6,	// Write mag data
		DEFINE_MAG_DATA			=0xB7,	// Define mag data
		GET_HICO_FLAG			=0xB8,	// Get hico flag
		SET_HICO_FLAG			=0xB9,	// Set hico flag
		/**************************************************************************
		* Chip Commands
		**************************************************************************/
		CHIP_WRITE				=0xBA,	// Chip => Write only Data to ICC
		CHIP_TRANSFER			=0xBB,	// Chip => Transfer Data with ICC
		CHIP_SELECTION			=0xBC,	// Chip / SAM selector
		MEMCARD_CMD				=0xBD,	// Memory cards => Data transfer
		MEMCARD_SET				=0xBE,	// Memory cards => Select Type of card
		CHIP_ACTIVATE			=0xBF,	// Chip => Activation: Cold/Warm reset
		CHIP_DEACTIVATE			=0xC0,	// Chip => Deactivation
		/**************************************************************************
		* Transport Commands
		**************************************************************************/
		MOVE					=0xC1,	// Move to desired position
		POSITION				=0xC2,	// Returns switch status byte
		LOCK					=0xC3,	// Card Locking
		CAPTURE_CARD			=0xC4,	// Card captured to rear side
		CHIP_POS				=0xC5,	// Card to chip position
		UNLOCK					=0xC6,	// Card Unlocking
		ABORT					=0xC7,	// Abort 'Arm to read' State
		ARM						=0xC8,	// Arm: Clear buffer, wait for Card
		ARM_DEBUG				=0xC9,	// Arm: Clear buffer, wait for Card		
		INTAKE					=0xCA,	// Intake to given position   
		ABORT_INTAKE			=0xCB,	// Abort Intake

        PRINT                   =0xCD,  // Print
        NFC_CMD                 =0x4F,  // NFC command
		NFC_INIT                =0xD0,  // Init NFC device
		NFC_TARGET              =0xD1   // Listen NFC device
	};

	/// <summary>
	/// Zusammenfassung fr Thread882.
	/// </summary>
	public class Link882
	{
        public Link882()
		{
			//
			// TODO: Fgen Sie hier die Konstruktorlogik hinzu
			//
		}
		public static int netAddress=1;
		public static EventFunction evFunction = null;
		public static BootEvent bCall = null;
		public static FmEvent fmCall = null;
		public static cmdCode lastCommand = 0;
		public static retCode lastEvent = 0;
		public static retCode lastCommandStatus = 0;
		public static byte[] cs_rcvBuffer = new byte[1040];
		public static int cs_rcvBufferLen=0;
		public static byte[] ev_rcvBuffer = new byte[1040];
		public static int ev_rcvBufferLen=0;
		public static bool tagInRange=false;
		public static byte tagInfo=0;
		public static bool tagCollision=false;
		public static int progAddress=0;
		public static bool skipMessage=false;
		public static bool removePICC=false;
				        						
		#region Callback Functions
		
		public delegate int FmEvent(int status, int val);
		
		public static int FmEventFunction(int status, int val)
		{
			if(status==2) progAddress=0;
			if(status==3 && progAddress==0) progAddress = val;
			//PutMessage("FM Status:" + status.ToString()+" Value:" + val.ToString(),2);

			if(progAddress!=0)
			{				
				if(status==3) 
				{
					frmMain.CoRMain.formProduction.displayStatus("Programming: 0x" + Convert.ToString(val,16).ToUpper(),0);
					frmMain.CoRMain.formBootROM.displayStatus("Programming: 0x" + Convert.ToString(val,16).ToUpper(),0);
					if(frmMain.CoRMain.formProduction.Created) frmMain.CoRMain.formProduction.setStatus(((progAddress-val)*100)/progAddress);
                    if (frmMain.CoRMain.formFlashLoader.Created) frmMain.CoRMain.formFlashLoader.setStatus(((progAddress - val) * 100) / progAddress);
					if(frmMain.CoRMain.formBootROM.Created) frmMain.CoRMain.formBootROM.setStatus(((progAddress-val)*100)/progAddress);	
				}
				else if(status==6)
				{
					frmMain.CoRMain.formProduction.displayStatus("Verifying: 0x" + Convert.ToString(val,16).ToUpper(),0);
					frmMain.CoRMain.formBootROM.displayStatus("Verifying: 0x" + Convert.ToString(val,16).ToUpper(),0);

					if(val<=progAddress)
					{
						if(frmMain.CoRMain.formProduction.Created) frmMain.CoRMain.formProduction.setStatus((val*100)/progAddress);						
						if(frmMain.CoRMain.formBootROM.Created) frmMain.CoRMain.formBootROM.setStatus((val*100)/progAddress);						
					}
				}
			}
			return 1;
		}

		public delegate void BootEvent(int percent);
		
		public static void Boot_RxEvent(int percent)
		{
			// This function will be called during the firmware update
			// to update the progress bar
			if(frmMain.CoRMain.formBoot882.Created) frmMain.CoRMain.formBoot882.setProgressBar(percent);	
			if(frmMain.CoRMain.formProduction.Created)frmMain.CoRMain.formProduction.setStatus(percent);
		}	
		
		public delegate void EventFunction(int addr,int status, IntPtr data, int len);
		
		public static void RxEventFunction(int addr,int status, IntPtr data, int len)
		{
			string sadr,sstat,sdatarcv,srcvlen;
            if (len == -1)
            {
                PutMessage("Data Len = -1!", 0);
                return;
            }
			Trace.WriteLine("Rx Event");
			// Status bytes, equal or greater then 0x30 are event messages
			if((status >=0x30 && status != Convert.ToInt32(retCodeLegacy.TYPEAPICC) && status!= Convert.ToInt32(retCodeLegacy.TYPEBPICC))
				|| ((status==Convert.ToInt16(retCodeLegacy.PICC_ACK)) && len>0)
				|| ((status==Convert.ToInt16(retCodeLegacy.REMOVED) && removePICC)) )
			{
				removePICC=false;
				// Copy incoming data to global buffer
				Marshal.Copy(data,ev_rcvBuffer,0,len);
				// Received length
				ev_rcvBufferLen = len;
				// Prepare data to display
				sadr = Utility.intToString(addr);
				sstat = "," + Utility.intToString(status);
				sdatarcv = ",0x" + Utility.DecodeHexl(ev_rcvBuffer,len);
				srcvlen = "," + Utility.intToString(len);

				// event occoured
				PutMessage("Event( "+ sadr + sstat+sdatarcv+srcvlen+" ); Status: " + Utility.DecodeStatus(Convert.ToByte(status)),7);

				// Is this event a switch report?
				if(status==Convert.ToInt16(retCode.event_SWITCH_REPORT))
				{
					// Report the switch change
					if(frmMain.CoRMain.formMaintenance.Created)
					{
						// Update the switch status textbox in the maintenance form
						frmMain.CoRMain.formMaintenance.setStatusText(Utility.DecodeHexl(ev_rcvBuffer,2));				
					}

					// Report the switch change
					if(frmMain.CoRMain.form910Premium.Created)
					{
						// Update the switch status textbox in the maintenance form
						frmMain.CoRMain.form910Premium.setStatusText(Utility.DecodeHexl(ev_rcvBuffer,2));				
					}
                    
					// Report the switch change
					if(frmMain.CoRMain.form910Basic.Created)
					{
						// Update the switch status textbox in the maintenance form
						frmMain.CoRMain.form910Basic.setStatusText(Utility.DecodeHexl(ev_rcvBuffer,2));				
					}

					// Report the switch change
					if(frmMain.CoRMain.form900.Created)
					{
						// Update the switch status textbox in the maintenance form
						frmMain.CoRMain.form900.setStatusText(Utility.DecodeHexl(ev_rcvBuffer,2));				
					}
				}
				else if((status==Convert.ToInt16(retCode.event_ARM_ACK)) || (status==Convert.ToInt16(retCode.event_ARM_NACK)))
				{
					lastEvent = (retCode)status;

					if(frmMain.CoRMain.formMagstripe.Created)
					{
						frmMain.CoRMain.formMagstripe.ClearStat();
						frmMain.CoRMain.formMagstripe.SetStatus1(ev_rcvBuffer[0]);
						frmMain.CoRMain.formMagstripe.SetStatus2(ev_rcvBuffer[1]);
						frmMain.CoRMain.formMagstripe.SetStatus3(ev_rcvBuffer[2]);
					}
                    if (frmMain.CoRMain.form900.Created)
                    {
                        frmMain.CoRMain.form900.setAck(status);   
                    }
				}
                else if ((status == Convert.ToInt16(retCode.event_INTAKE_ACK)) || (status == Convert.ToInt16(retCode.event_INTAKE_NACK)))
                {
                    lastEvent = (retCode)status;

                    if (frmMain.CoRMain.form900.Created)
                    {
                        frmMain.CoRMain.form900.setAck(status);
                    }
                }
				else if(status==Convert.ToInt16(retCode.event_PICC_ACK) || (status== Convert.ToInt16(retCodeLegacy.PICC_ACK)))
				{
					tagInRange = true;
					tagInfo = ev_rcvBuffer[0];
					if((ev_rcvBuffer[0] & 0x08)==0x08)
					{
						tagCollision = true;
						frmMain.CoRMain.formLoopback.PutMessage("Collision detected!");
					}
					else
						tagCollision = false;
				}
				else if(status==Convert.ToInt16(retCode.event_REMOVED) || (status==Convert.ToInt16(retCodeLegacy.REMOVED)))
				{
					tagInfo = 0;
					tagInRange = false;
				}
				else if(status==Convert.ToInt16(retCode.event_KEY_STROKE))
				{
					// Report the switch change
					if(frmMain.CoRMain.formMaintenance.Created)
					{
						// Update the key textbox in the maintenance form
						frmMain.CoRMain.formMaintenance.setEnteredKey(ev_rcvBuffer[0]);				
					}
				}
				else
					lastEvent = (retCode)status;
			}
			else	// Status bytes, less then 0x30 are response messages
			{
				// Copy incoming data to global buffer
				Marshal.Copy(data,cs_rcvBuffer,0,len);
				// Received length
				cs_rcvBufferLen = len;
				// Prepare data to display
				sadr = Utility.intToString(addr);
				sstat = "," + Utility.intToString(status);
				sdatarcv = ",0x" + Utility.DecodeHexl(cs_rcvBuffer,len);
				srcvlen = "," + Utility.intToString(len);

				// Answer
				if(lastCommand != 0)
				{
					if(status==0 || status == Convert.ToInt32(retCodeLegacy.TYPEAPICC) || status== Convert.ToInt32(retCodeLegacy.TYPEBPICC))
						PutMessage(Convert.ToString(lastCommand)+ " -> Response( "+ sadr + sstat+sdatarcv+srcvlen+" ); Status: " + Utility.DecodeStatus(Convert.ToByte(status)) + " ASCII: " + Utility.DecodeAsciil(cs_rcvBuffer,len),1);			
					else
						PutMessage(Convert.ToString(lastCommand)+ " -> Response( "+ sadr + sstat+sdatarcv+srcvlen+" ); Status: " + Utility.DecodeStatus(Convert.ToByte(status)),0);			
				}
				else
				{
					PutMessage("Response( "+ sadr + sstat+sdatarcv+srcvlen+" ); Status: " + Utility.DecodeStatus(Convert.ToByte(status)) + " ASCII: " + Utility.DecodeAsciil(cs_rcvBuffer,len),1);			
				}
				// Last command has received its answer
				lastCommand = 0;
				// Save the status of the last command
				lastCommandStatus = (retCode)status;
			}
		}
		#endregion

		#region DLL_IMPORT
		[DllImport("LINK882.dll")]
		public static extern byte sendCmd(int add, int cmd,[Out()] byte[]  snd, int len);  
		[DllImport("LINK882.dll")]
		public static extern byte sendCommBytes([Out()] byte[]  snd, int len);  
		[DllImport("LINK882.dll")]
		public static extern bool initComm(int port,int baud,int timeout);
		[DllImport("LINK882.dll")]
		public static extern bool setCallBackPointer(EventFunction evFunction);
		[DllImport("LINK882.dll")]
		public static extern bool rtsControl(int rts);
		[DllImport("LINK882.dll")]
		public static extern bool dtrControl(int rts);
		[DllImport("LINK882.dll")]
		public static extern bool killComm();
		[DllImport("LINK882.dll")]
		public static extern bool getDLLVersion([Out()] byte[] ver);
		[DllImport("LINK882.dll")]
		public static extern byte tcpConnect([Out()] byte[] data,int port);
		[DllImport("LINK882.dll")]
		public static extern byte tcpSend([Out()] byte[] buffer, int len,[Out()] byte[] rbuffer,[Out()] byte[] rxlen);
		[DllImport("LINK882.dll")]
		public static extern byte tcpClose();
		[DllImport("LINK882.dll")]
		public static extern byte udpOpen(int port, [Out()] byte[] dest_addr, int listen_only);
		[DllImport("LINK882.dll")]
		public static extern byte udpSendTo([Out()] byte[] addr, int port ,[Out()] byte[] buffer, int len);
		[DllImport("LINK882.dll")]
		public static extern byte udpClose();
		[DllImport("LINK882.dll")]
		public static extern byte usbHidConnect([Out()] byte[] name);
		[DllImport("LINK882.dll")]
		public static extern byte usbHidKill();

		[DllImport("LINK882.dll")]
		public static extern byte Boot882_ProgFirmware(string file);
		[DllImport("LINK882.dll")]
		public static extern byte Boot882_VerifyFirmware(string file);		
		[DllImport("LINK882.dll")]
		public static extern byte Boot882_EraseFlash();
		[DllImport("LINK882.dll")]
		public static extern byte Boot882_BlankCheck();
		[DllImport("LINK882.dll")]
		public static extern byte Boot882_Program(int offset, int length, [Out()] byte[] data);
		[DllImport("LINK882.dll")]
		public static extern byte Boot882_UserCodeCheck();
		[DllImport("LINK882.dll")]
		public static extern byte Boot882_ExeUserCode();
		[DllImport("LINK882.dll")]
		public static extern byte Boot882_ExeBootCode();
		[DllImport("LINK882.dll")]
		public static extern byte Boot882_ReadFlash(uint offset, uint length, [Out()] byte[] data);
		[DllImport("LINK882.dll")]
		public static extern byte Boot882_SetCallBackPointer(BootEvent bCall);
		[DllImport("LINK882.dll")]
		public static extern byte Boot882_GetMode();
		[DllImport("LINK882.dll")]
		public static extern byte Boot882_SetAddr(int adr);
		[DllImport("LINK882.dll")]
		public static extern byte Boot882_SignFirmware();
		[DllImport("LINK882.dll")]
		public static extern int chksum_crc32 ([Out()] byte[] block, uint length);

		[DllImport("LINK882.dll")]
		public static extern int bootProg(string hFile); 
		[DllImport("LINK882.dll")]
		public static extern int bootVerify(string hFile); 
		[DllImport("LINK882.dll")]
		public static extern int bootConnect(int com, int baud, int uc); // 0: lpc2364, 1: lpc1754
		[DllImport("LINK882.dll")]
		public static extern int bootDisconnect(); 
		[DllImport("LINK882.dll")]
		public static extern int bootErase(); 
		[DllImport("LINK882.dll")]
		public static extern bool FM_SetCallBackPointer(FmEvent evFunction);

		#endregion
		
		public static byte sendCmd882(cmdCode command, byte[] sndBuffer, int sndBufferLen)
		{
			byte ret=0;
			string sadr,scmd,sdatasnd,ssndlen;
						
			try
			{
				if(Param.Interface == 1)
				{
					// Store last executed command
					lastCommand = command;
				}
				// Send command
				Trace.WriteLine("Send Cmd");
				ret = sendCmd(netAddress,Convert.ToInt16(command),sndBuffer,sndBufferLen);

				// Prepare data to display				
				sadr = Utility.intToString(netAddress);
				scmd = ",<" + command.ToString() + ">";
				sdatasnd = ",0x" + Utility.DecodeHexl(sndBuffer,sndBufferLen);
				ssndlen = "," + Utility.intToString(sndBufferLen);
			
				if(ret==0 ) 
				{
					if(Param.Interface != 1)
					{
						// Store last executed command
						lastCommand = command;
					}
					// Clear status
					lastCommandStatus = 0;
					PutMessage("sendCmd(" + sadr + scmd + sdatasnd + ssndlen + "); Status: " + Utility.DecodeStatus(ret), 5);
				}
				else
					PutMessage("sendCmd(" + sadr + scmd + sdatasnd + ssndlen + "); Status: " + Utility.DecodeStatus(ret), 6);

				return ret;
			}
			catch
			{
				// Exception
				PutMessage("Windows Error",3);
				return Convert.ToByte(retCode.stat_ERROR);
			}
		}
		
		public static void PutMessage(string Message, int tf)
		{
			if(skipMessage)return;
			// Add message to history window
			frmMain.CoRMain.PutMessage(Message,tf);		
		}

		public static bool waitAnswer(int timeout)
		{
			long TimerCount = System.Environment.TickCount;
			// wait for answer or timeout
			while( ( System.Environment.TickCount-TimerCount ) < timeout)
			{
				// Do other things
				Application.DoEvents();
				if(lastCommand==0)
				{
					// Check status byte
					if(lastCommandStatus == retCode.stat_OK)
						return true;	
					else
						return false;
				}
			}
			// Timeout occured
			PutMessage("Timeout error",3);
			lastCommand=0;
			lastCommandStatus=0;
			return false;
		}

		public static bool waitEventACK(int timeout)
		{
			long TimerCount = System.Environment.TickCount;
			// wait for answer or timeout
			while( ( System.Environment.TickCount-TimerCount ) < timeout)
			{
				// Do other things
				Application.DoEvents();
				if(lastEvent>0)
				{
					// Check status byte
					if(lastEvent == retCode.event_ARM_ACK || lastEvent == retCode.event_INTAKE_ACK)
						return true;	
					else
						return false;
				}
			}
			// Timeout occured
			PutMessage("Timeout error",3);
			lastEvent=0;
			return false;
		}

		public static bool waitEventStart(int timeout)
		{
			long TimerCount = System.Environment.TickCount;
			// wait for answer or timeout
			while( ( System.Environment.TickCount-TimerCount ) < timeout)
			{
				// Do other things
				Application.DoEvents();
				if(lastEvent>0)
				{
					// Check status byte
					if(lastEvent == retCode.event_BOOT_MODE_START || lastEvent == retCode.event_USER_MODE_START)
						return true;	
					else
						return false;
				}
			}
			// Timeout occured
			PutMessage("Timeout error",3);
			lastEvent=0;
			return false;
		}
	}
}
