/**************************************************************************
***************************************************************************
---------------------------------------------------------------------------
*		C O M M O N - P R O T O C O L
---------------------------------------------------------------------------
*		Id:			protCommon.h, 21/04/2010, by hhosoglu
*		Compiler:	RealView C Compiler V 4.0
*		Processor:	LPC23xx ARM7 32-Bit uC
*		Revision:	V1.11
---------------------------------------------------------------------------
*		This code is a part of the ddm library
*		Copyright (c) 2006-2009 ddm Software
---------------------------------------------------------------------------
***************************************************************************
**************************************************************************/
#ifndef __PROTCOMMON__H
#define __PROTCOMMON__H
/*************************************************************************/
/*************************************************************************/
/**************************************************************************
* Contactless ISO14443 PCD Commands
**************************************************************************/
#define cmd_PCD_KILL				0x1F	// Reset PCD	
#define cmd_PCD_TYPEA_INIT			0x20	// Init PCD	
#define cmd_PCD_TYPEB_INIT			0x50	// Initialize Type B interface
#define cmd_PCD_WRITEE2				0x21	// Get data from FIFO and write it to the E2PROM
#define cmd_PCD_READE2				0x22	// Read data from E2PROM and put it into the FIFO
#define cmd_PCD_RESETPHASE			0x23	// Runs the Reset- and Initialisation Phase
#define cmd_PCD_RF_RESET			0x25	// Rf reset
#define cmd_PCD_RF_OFF				0x26	// Rf off
#define cmd_WRITERC					0x57	// Write rc
#define cmd_READRC					0x58	// Read rc
#define cmd_PCD_SET_ATTRIB			0x29	// Set attribute
#define cmd_PICC_EXCHANGE_BLOCK	0x2A	// Exchange block 
#define cmd_PCD_ACTIVATE_ANT		0x61	// Activate antenna 1 or 2
#define cmd_PICC_REMOVE				0x62	// Remove a picc
/**************************************************************************
* Contactless ISO14443 TypeA mifare Commands
**************************************************************************/
#define cmd_PICC_AUTHENT			0x13	// Authentication
#define cmd_PICC_AUTHENTKEY			0x14	// Authentication with extern keys
#define cmd_PICC_READ				0x15	// Read 16 byte block
#define cmd_PICC_WRITE				0x16	// Write 16 byte block
#define cmd_PICC_WRITE4				0x17	// Write 4 byte block
#define cmd_PICC_VALUE				0x18	// Decrement value
#define cmd_PICC_PERSO_UID			0x1A	// Personalize UID Usage
#define cmd_PICC_TRANSFER			0x1B	// Transfer command code
#define cmd_PCD_LOADKEY				0x1D	// Read a master key from the FIFO and put it into the master key buffer 	
#define cmd_PCD_LOADKEYE2			0x1E	// Read a master key from the E2PROM and put it into the master key buffer
/**************************************************************************
* Contactless ISO14443 TypeA Commands
**************************************************************************/
#define cmd_PICC_REQUEST			0x10	// Request idle
#define cmd_PICC_ANTICOLL			0x11	// Anticollision
#define cmd_PICC_SELECT				0x12	// Select given card
#define cmd_GETCARDID				0x24	// This command performs the Req,Anticoll, and select operations
#define cmd_PICC_ANTICOLLSEL		0x19	// Anticollselect
#define cmd_PICC_HALTA				0x1C	// Halt
#define cmd_PICC_ACTIVATE_IDLE		0x59	// Activate idle card
#define cmd_PICC_ACTIVATE_WAKEUP	0x5A	// Wake up a halted card
#define cmd_PICC_ACTIVATE_AB_WAKEUP	0x7C	// Test command
#define cmd_PICC_DO_PPS				0x2B	// Protocol Parameter Selection
#define cmd_PICC_REQUEST_ATS		0x3A	// Request ATS
/**************************************************************************
* Contactless ISO14443 TypeB Commands
**************************************************************************/
#define cmd_PICC_REQUESTB			0x51	// Request or wake up a Type B picc
#define cmd_PICC_SLOTMARKER			0x5F	// Slot Marker
#define cmd_PICC_ATTRIB				0x52	// Attrib
#define cmd_PICC_ATTRIB_HBR			0x2C	// Attrib Higher Bautrate
#define cmd_PICC_HALTB				0x53	// Halt Type B picc
/**************************************************************************
* Contactless ISO14443-4 T=CL Commands
**************************************************************************/
#define cmd_PCD_SET_FWI      		0x0F 	// Set Frame Waiting Time Integer
#define cmd_PICC_DETECT				0x5B	// Detect picc
#define cmd_PICC_ACTIVATE			0x5C	// Activate picc
#define cmd_PICC_SEND_BLOCK			0x54	// Send block to the picc
#define cmd_PICC_SEND_BLOCK_BEEP	0x4E	// Send block and beep
#define cmd_PICC_SEND_ACK			0x55	// Send ack or nack
#define cmd_PICC_SEND_REQ			0x56	// Send WTX or DSEL
#define cmd_PICC_DESELECT			0x86	// Perform Deselect
/**************************************************************************
* Reader Commands
**************************************************************************/
#define cmd_MAINTENANCE				0x27	// Maintenance command
#define cmd_READ_STATUS				0x28	// Transmits reader status bytes
#define cmd_READ_SWITCH				0x2D	// Read switch status
#define cmd_SET_BAUDRATE			0x2F	// Change baudrate
#define cmd_READ_REGISTER			0x30	// Read module register
#define cmd_WRITE_REGISTER			0x31	// Update module register
#define cmd_MODULE_RESET			0x33	// Reset
#define cmd_SET_RUNNING_MODE		0x5D	// Set running mode
#define cmd_GET_INFO				0x72	// Get module information data
#define cmd_IAP						0x2E	// IAP commands
/**************************************************************************
* Peripheral Function Commands
**************************************************************************/
#define cmd_WRITE_LCD				0x32	// Update dot-matrix lcd
#define cmd_ACT_RELAY				0x40	// Activate relay
#define cmd_GET_INPUT				0x87	// Scan input pins
#define cmd_SET_RELAY				0x88	// Set relay outputs
#define cmd_SET_LED					0x41	// Set or clear RED/GREEN leds
#define cmd_SET_BUZZER				0x42	// Set buzzer
#define cmd_RTC_UPDATE				0x44	// Update date time
#define cmd_SET_SHUTTER				0x45	// Set shutter
#define cmd_MEMORY_WRITE			0x46	// External eeprom write
#define cmd_MEMORY_READ				0x47	// External eeprom read
#define cmd_FRM_MEM_WRITE			0x34	// Program firmware data
#define cmd_FRM_MEM_READ			0x35	// Read firmware data
#define cmd_FRM_MEM_ERASE			0x36	// Erase firmware regien
#define cmd_RTC_READ				0x48	// Read date time info
#define cmd_MEMORY_ERASE			0x4D	// External flash sektor erase
#define cmd_SET_PPLEDS				0x5E	// Set indicator leds
#define cmd_INDICATE_PP				0x60	// Visual and audio indicators
#define cmd_SET_AC					0x71	// Set access control bit
#define cmd_GRAPHIC_LCD_INIT		0x73	// Init graphic lcd
#define cmd_GRAPHIC_LCD_GOTO		0x74	// goto dot position
#define cmd_GRAPHIC_LCD_WRITE		0x75	// write on graphic lcd
#define cmd_GRAPHIC_LCD_SETDOT		0x76	// set dot
#define cmd_GRAPHIC_LCD_CLRDOT		0x77	// clear dot
#define cmd_GRAPHIC_SAVE_IMAGE		0x78	// save 1K image in ext. flash
#define cmd_GRAPHIC_LOAD_IMAGE		0x79	// load 1K image data direct
#define cmd_GRAPHIC_CALL_IMAGE		0x7A	// call a image form ext. flash
#define cmd_GRAPHIC_CLEAR_IMAGES	0x7B	// erase all saved images
#define cmd_EXTENDED				0x43	// Extended commands
/**************************************************************************
* Access Control Function Commands
**************************************************************************/
#define cmd_GET_MEM_INFO			0x49	// Get memory status for AC
#define cmd_SAVE_LIST				0x4A	// Save ID list
#define cmd_GET_LIST				0x4B	// Get ID list
#define cmd_GET_ACTIVITY_FILE		0x4C	// Get activity file
#define cmd_ERASE_ACTIVITY_FILE		0x65	// Erase activity file
#define cmd_ERASE_ID				0x66	// Erase an ID from the list
#define cmd_ACK						0x67	// Send ack
#define cmd_NACK					0x68	// Send nack
/**************************************************************************
* MDB Commands
**************************************************************************/
#define cmd_SET_AVAILABLE_FUND		0x7D	// MDB set fund
#define cmd_SET_VENDING_STATUS		0x7E	// MDB set vending status
#define cmd_GET_VMC_CONFIG			0x7F	// MDB get vmc configuration
/**************************************************************************
* Keypad Commands
**************************************************************************/
#define cmd_KEYPAD_VERSION			0x80	// Read keypad Version
#define cmd_EN_KEYPAD				0x81	// Enable keypad scan
#define cmd_DIS_KEYPAD				0x82	// Disable keypad scan
#define cmd_GET_KEYSTROKES			0x83	// get keystrokes
#define cmd_EN_AUTOREPORT			0x84	// enable auto report
#define cmd_DIS_AUTOREPORT			0x85	// disable auto report
/**************************************************************************
* 900 Dispenser Commands
**************************************************************************/
#define cmd_DISPENSER_DO_MOVE		0x90	// Move a card to the defined position
#define cmd_DISPENSER_SWITCH_STAT	0x91	// Read switch status
#define cmd_DISPENSER_MAINT			0x92	// Maintenance command
/**************************************************************************
* Bootloader Commands
**************************************************************************/
#define BOOT_ERASE					0xA0	// Erase user code
#define BOOT_BLANK_CHECK			0xA1	// Blank check
#define BOOT_PROGRAM				0xA2	// Program flash
#define BOOT_READ					0xA3	// Read from flash
#define BOOT_VERSION				0xA4	// Read Version
#define BOOT_RESET					0xA5	// Reset
#define BOOT_EXE_USER_CODE			0xA6	// Jump to user code
#define BOOT_CHECK_USER_CODE		0xA7	// Check user codes exists?
#define BOOT_EXE_BOOT_CODE			0xA8	// Jump to boot code
#define GET_MODE					0xA9	// Get current mode
/**************************************************************************
* Magstripe Read/Write Commands
**************************************************************************/
#define cmd_MAG_READ				0xB0	// Read magstripe
#define cmd_MAG_READ_DETAILED		0xB1	// Read detailed
#define cmd_MAG_GET_ISO_TRK			0xB2	// Transmit iso track
#define cmd_MAG_GET_ISO_FMT			0xB3	// Transmit ISO formated data
#define cmd_MAG_GET_CUSTOM_TRK		0xB4	// Transmit custom formated data
#define cmd_MAG_ERROR				0xB5	// Transmit error data
#define cmd_MAG_WRITE				0xB6	// Write mag data
#define cmd_DEFINE_MAG_DATA			0xB7	// Define mag data
#define cmd_GET_HICO_FLAG			0xB8	// Get hico flag
#define cmd_SET_HICO_FLAG			0xB9	// Set hico flag
/**************************************************************************
* Chip Commands
**************************************************************************/
#define cmd_CHIP_WRITE			0xBA	// Chip => Write only Data to ICC
#define cmd_CHIP_TRANSFER		0xBB	// Chip => Transfer Data with ICC
#define cmd_CHIP_SELECTION		0xBC	// Chip / SAM selector
#define cmd_MEMCARD_CMD			0xBD	// Memory cards => Data transfer
#define cmd_MEMCARD_SET			0xBE	// Memory cards => Select Type of card
#define cmd_CHIP_ACTIVATE		0xBF	// Chip => Activation: Cold/Warm reset
#define cmd_CHIP_DEACTIVATE		0xC0	// Chip => Deactivation
#define cmd_PRINT				0xCD	// Printer command
/**************************************************************************
* NFC Commands
**************************************************************************/
#define cmd_NFC_CMD				0x4F	// Nfc commands
/**************************************************************************
* Transport Commands
**************************************************************************/
#define cmd_MOVE					0xC1	// Move to desired position
#define cmd_POSITION				0xC2	// Returns switch status byte
#define cmd_LOCK					0xC3	// Card Locking
#define cmd_CAPTURE_CARD			0xC4	// Card captured to rear side
#define cmd_CHIP_POS				0xC5	// Card to chip position
#define cmd_UNLOCK					0xC6	// Card Unlocking
#define cmd_ABORT_ARM				0xC7	// Abort 'Arm to read' State
#define cmd_ARM						0xC8	// Arm: Clear buffer, wait for Card
#define cmd_ARM_DEBUG				0xC9	// Arm: Clear buffer, wait for Card
#define cmd_INTAKE					0xCA	// Intake to given position
#define cmd_ABORT_INTAKE			0xCB	// Abort Intake
/**************************************************************************
**************************************************************************/
// 0x01-0x0E
// 0x37-0x39
// 0x3B-0x3F
// 0x63-0x64
// 0x69-0x70
// 0x89-0x8F
// 0x94-0x9F
// 0xAA-0xAF
// 0xCD-....
/**************************************************************************
**************************************************************************/

/**************************************************************************
* R E T U R N   S T A T U S   C O D E S		0x0..
**************************************************************************/
#define stat_OK						0x00	// status OK
#define stat_NO_TAG_ERR				0x01	// no tag error		 	
#define stat_COLL_ERR				0x02	// collision error				
#define stat_AUTH_ERR				0x03	// mifare authentication error		
#define stat_PROTOCOL_ERR			0x04	// T=CL protocol error
#define stat_TRANSMISSION_ERR		0x05	// T=CL transmission error
#define stat_TIMEOUT_ERR			0x06	// T=CL timeout error
#define stat_BUFFER_OVERFLOW_ERR	0x07	// buffer overflow error
#define stat_ADR_OVERFLOW_ERR		0x08	// address overflow error
#define stat_UNKNOWN_CMD			0x09	// unknown command
#define stat_ERROR					0x0A	// uncategorized error
#define stat_COMM_TIMEOUT			0x0B	// communication timeout
#define stat_USB_DEVICE_ERROR		0x0C	// usb device error
#define stat_BOOT_ERROR				0x0D	// boot error
#define stat_BOOT_OVERFLOW			0x0E	// boot overflow error
#define stat_BOOT_TIMEOUT			0x0F	// boot timeout error
#define stat_NO_USER_CODE			0x10	// no user code exists
#define stat_INVALID				0x11	// invalid 
#define stat_NO_DATA				0x12	// No magnetic data decoded
#define stat_UNAVAILABLE			0x13	// cannot perform this command
#define stat_MAG_DATA_NOT_VALID		0x14	// magdata not valid
#define stat_CARD_JAM				0x15	// card jam
#define stat_BCC_Error				0x16	// Bcc error
#define stat_XC_TOUT				0x17	// XC Timout
#define stat_NFC_ERR				0x18	// NFC interface error
#define stat_MAG_ERR				0x19	// Mag head decode err
/**************************************************************************
* E V E N T   S T A T U S   C O D E S		0x3..
**************************************************************************/
#define event_REMOVED				0x30	// contactless tag is removed
#define event_PICC_ACK				0x31	// contactless tag is detected
#define event_USER_MODE_START		0x32	// user mode started
#define event_BOOT_MODE_START		0x33	// boot mode started
#define event_SWITCH_REPORT			0x34	// automatic switch report
#define event_MEM_CARD_ACTIVATED	0x35	// memory card is activated	
#define event_CHIP_CARD_ACTIVATED	0x36	// auto atr
#define event_MDB_DATA				0x37	// MDB data packet
#define event_MDB_STATE				0x38	// MDB state change
#define event_KEY_STROKE			0x39	// Key entered
#define event_ARM_ACK				0x3A	// Arm to read ack
#define event_ARM_NACK				0x3B	// Arm to read nack
#define event_NO_MAG_CARD			0x3C	// No magnetic tracks detected
#define event_INTAKE_ACK			0x3D	// Intake to position ack	
#define event_INTAKE_NACK			0x3E	// Intake to position not ack	
#define event_PICC_PPSE				0x3F	// contactless tag is detected and SelectPPSE is performed
#endif
